package com.mt.wms.empty.task;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.*;
import com.mt.wms.core.dal.service.*;
import com.mt.wms.core.utils.HttpClient;
import com.mt.wms.core.utils.IDGenerator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @Author: liguanghao
 * @Date: 2021/11/18 20:27
 * @Version 1.0
 */
@Service
@Transactional
public class AsynRunTaskService extends BaseService {
    private final static Logger logger = LoggerFactory.getLogger(AsynRunTaskService.class);

    @Resource
    private CurrTaskServiceBiz currTaskServiceBiz;
    @Resource
    private RunTaskServiceBiz runTaskServiceBiz;
    @Resource
    private LocationServiceBiz locationServiceBiz;
    @Resource
    private InStockInfoServiceBiz inStockInfoServiceBiz;
    @Resource
    private InStockInfoHisServiceBiz inStockInfoHisServiceBiz;
    @Resource
    private TaskDistanceUtils taskDistanceUtils;
    @Resource
    private VehicleInfoServiceBiz vehicleInfoServiceBiz;
    @Resource
    private PointInfoServiceBiz pointInfoServiceBiz;
    @Resource
    private KilnInfoServiceBiz kilnInfoServiceBiz;
    @Resource
    private PlcNameSpaceServiceBiz plcNameSpaceServiceBiz;
    @Resource
    private ParTemValueServiceBiz parTemValueServiceBiz;
    @Resource
    private ParEleValueServiceBiz parEleValueServiceBiz;
    @Resource
    private ParRotSpeedValueServiceBiz parRotSpeedValueServiceBiz;
    @Resource
    private ParGasValueServiceBiz parGasValueServiceBiz;
    //窑炉可用未满的情况下，调用车辆起点为提升台终点为窑炉
    @Async("asyncServiceExecutor")
    public void asynRunTask(Long currTaskId,Long vehicleId){
        CurrTask currTask = currTaskServiceBiz.getById(currTaskId);
        VehicleInfo vehicle = vehicleInfoServiceBiz.getById(vehicleId);
        Integer otherNowPoint=0;
        if (vehicleId==1){
            // TODO: 2022/2/26 获取RGV2当前位置
            otherNowPoint=1;
            //两辆小车是否冲突
            Boolean conflictBoolean = taskDistanceUtils.conflictForVehicle(vehicleId, currTask.getStartPosition(), currTask.getTargetPosition(), otherNowPoint);
            while (conflictBoolean){
                //若冲突
                // TODO: 2022/2/26  执行小车移位任务。移位任务完成后再判断是否冲突,移位任务可以直接让小车移位到两端
                Map<String, Integer> json = new HashMap();
                //type=1为 入
                // taskType 4 单移动
                json.put("taskType", 4);
                json.put("sendRow", 39);
                String taskJson = JSON.toJSONString(json);
                try {
                    HttpClient.httpPost("http://localhost:8009/rgv2/sendTask",taskJson);
                } catch (IOException e) {
                    e.printStackTrace();
                    logger.error(new Date() + "WCS接口超时未响应！");
                }
            }
        }else {
            //获取RGV1当前位置
            otherNowPoint=2;
            //两辆小车是否冲突
            Boolean conflictBoolean = taskDistanceUtils.conflictForVehicle(vehicleId, currTask.getStartPosition(), currTask.getTargetPosition(), otherNowPoint);
            while (conflictBoolean){
                //若冲突
                // TODO: 2022/2/26  执行小车移位任务。移位任务完成后再判断是否冲突,移位任务可以直接让小车移位到两端
                Map<String, Integer> json = new HashMap();
                // taskType 4 单移动
                json.put("taskType", 4);
                json.put("sendRow", 1);
                String taskJson = JSON.toJSONString(json);
                try {
                    HttpClient.httpPost("http://localhost:8009/rgv1/sendTask",taskJson);
                } catch (IOException e) {
                    e.printStackTrace();
                    logger.error(new Date() + "WCS接口超时未响应！");
                }
            }
        }

        //新建一条执行任务的关系表存放任务执行信息
        RunTask runTask=new RunTask();
        runTask.setTaskId(currTask.getId());
        runTask.setTaskCode(currTask.getTaskCode());
        runTask.setVehicleId(vehicleId);
        runTask.setStartPosition(currTask.getStartPosition());
        runTask.setEndPosition(currTask.getTargetPosition());
        setCommonField(runTask);
        runTask.setBeginTime(LocalDateTime.now());
        runTaskServiceBiz.save(runTask);
        // TODO: 2021/12/14  调用车辆填入起终点,得到车辆执行结果
        Long startPoint = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, runTask.getStartPosition())).getCode();
        Long endPoint = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, runTask.getEndPosition())).getCode();
        Map<String, Object> json = new HashMap();
        // taskType 1 搬运
        json.put("taskType", 1);
        json.put("sendRow", endPoint);
        json.put("pickRow",startPoint);
        json.put("taskNo",currTaskId);
        json.put("ideNumber",1);
        json.put("processNumber",currTask.getPlcValue());
        String taskJson = JSON.toJSONString(json);
        String result =null;
        try {
            if (vehicleId==1){
                result = HttpClient.httpPost("http://localhost:8009/rgv1/sendTask", taskJson);
            }
            if (vehicleId==2){
                result = HttpClient.httpPost("http://localhost:8009/rgv2/sendTask", taskJson);
            }
        } catch (IOException e) {
            e.printStackTrace();
            logger.error(new Date() + "WCS接口超时未响应！");
        }
        JSONObject jsonObject = JSON.parseObject(result);
        int status = Integer.parseInt(String.valueOf(jsonObject.get("msg")));
        //执行成功,托盘进炉
        if (status==1){
            //更新调度表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setEndTime(LocalDateTime.now());
            runTask.setStatus(2);
            runTaskServiceBiz.updateById(runTask);
            //更新currTask表状态
            currTask.setIsIn(1);
            currTask.setRunTaskId(runTask.getId());
            currTask.setUpdateTime(LocalDateTime.now());
            currTaskServiceBiz.updateById(currTask);
            //更新小车状态
            vehicle.setStatus(0);
            vehicleInfoServiceBiz.updateById(vehicle);
        }else {
            runTask.setStatus(3);
            runTask.setUpdateTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            logger.info("任务 "+currTask.getTaskCode()+" 车辆从液压台台到窑炉过程中失败。");
            // TODO: 2021/12/14 websocket推送到前端,小车管理界面添加一个重置小车状态的接口。
        }
    }

    // TODO: 2022/2/26 修改下面的 方法加上小车操作如上
    //窑炉可用未满的情况下，调用车辆起点为缓存区终点为窑炉
    public void asynRunTaskForStockToKiln(Long stockInfokId,Long vehicleId){
        InStockInfo inStockInfo = inStockInfoServiceBiz.getById(stockInfokId);
        Location location = locationServiceBiz.getById(inStockInfo.getLocationId());
        Long currTaskId = inStockInfo.getTaskId();
        CurrTask currTask = currTaskServiceBiz.getById(currTaskId);
        Integer otherNowPoint=0;
        if (vehicleId==1){
            // TODO: 2022/2/26 获取RGV2当前位置
            otherNowPoint=1;
        }else {
            //获取RGV1当前位置
            otherNowPoint=2;
        }
        //两辆小车是否冲突
        Boolean conflictBoolean = taskDistanceUtils.conflictForVehicle(vehicleId, location.getCode(), currTask.getTargetPosition(), otherNowPoint);
        while (conflictBoolean){
            //若冲突
            // TODO: 2022/2/26  执行小车移位任务。移位任务完成后再判断是否冲突,移位任务可以直接让小车移位到两端
        }

        //新建一条执行任务的关系表存放任务执行信息
        RunTask runTask=new RunTask();
        runTask.setTaskId(currTask.getId());
        runTask.setTaskCode(currTask.getTaskCode());
        runTask.setVehicleId(vehicleId);
        // TODO: 2022/2/16  起点为库位
        runTask.setStartPosition(location.getCode());
        runTask.setEndPosition(currTask.getTargetPosition());
        setCommonField(runTask);
        runTask.setBeginTime(LocalDateTime.now());
        runTaskServiceBiz.save(runTask);
        //修改缓存区存储情况表状态
        inStockInfo.setStatus(1);
        inStockInfoServiceBiz.updateById(inStockInfo);
        // TODO: 2021/12/14  调用车辆填入起终点,得到车辆执行结果

        //执行成功,托盘进炉
        if (true){
            //更新调度表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setEndTime(LocalDateTime.now());
            runTask.setStatus(2);
            runTaskServiceBiz.updateById(runTask);
            //更改缓存区存储状态,添加缓存区存储历史情况记录
            inStockInfo.setStatus(2);
            inStockInfoServiceBiz.updateById(inStockInfo);
            InStockInfoHis inStockInfoHis=new InStockInfoHis();
            BeanUtils.copyProperties(inStockInfo,inStockInfoHis);
            inStockInfoHisServiceBiz.save(inStockInfoHis);
            inStockInfoServiceBiz.removeById(inStockInfo);
            //更新currTask表状态
            currTask.setIsIn(1);
            currTask.setRunTaskId(runTask.getId());
            currTask.setUpdateTime(LocalDateTime.now());
            currTaskServiceBiz.updateById(currTask);
        }else {
            runTask.setStatus(3);
            runTask.setUpdateTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            logger.info("任务 "+currTask.getTaskCode()+" 车辆从提升平台到窑炉过程中失败。");
            // TODO: 2021/12/14 websocket推送到前端,执行失败后车辆在中途，缓存区存储情况怎么修改？
        }
    }
    //窑炉已满，调用车辆起点为提升台，终点为缓存区
    @Async("asyncServiceExecutor")
    public void asynRunTaskToWarehouse(Long currTaskId,Long vehicleId){
        CurrTask currTask = currTaskServiceBiz.getById(currTaskId);
        Location location = locationServiceBiz.getById(currTask.getLocationId());
        VehicleInfo vehicle = vehicleInfoServiceBiz.getById(vehicleId);
        Integer otherNowPoint=0;
        if (vehicleId==1){
            // TODO: 2022/2/26 获取RGV2当前位置
            otherNowPoint=1;
        }else {
            //获取RGV1当前位置
            otherNowPoint=2;
        }
        //两辆小车是否冲突
        Boolean conflictBoolean = taskDistanceUtils.conflictForVehicle(vehicleId, currTask.getStartPosition(), currTask.getTargetPosition(), otherNowPoint);
        while (conflictBoolean){
            //若冲突
            // TODO: 2022/2/26  执行小车移位任务。移位任务完成后再判断是否冲突,移位任务可以直接让小车移位到两端
        }
        PointInfo pointOfLocation = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, location.getCode()));
        //新建一条执行任务的关系表存放任务执行信息,终点为缓存区空闲库位
        RunTask runTask=new RunTask();
        setCommonField(runTask);
        runTask.setTaskId(currTask.getId());
        runTask.setTaskCode(currTask.getTaskCode());
        runTask.setVehicleId(vehicleId);
        runTask.setBeginTime(LocalDateTime.now());
        runTask.setStartPosition(currTask.getStartPosition());
        runTask.setEndPosition(location.getCode());
        runTaskServiceBiz.save(runTask);

        //添加一条库位详情到in_stock_info表
        InStockInfo inStockInfo=new InStockInfo();
        inStockInfo.setStatus(1);
        inStockInfo.setInterCode(IDGenerator.gen("HCT", "yyyyMMddHHmm", 2, "WAREHOUSE_CODE"));
        inStockInfo.setTaskId(currTask.getId());
        inStockInfo.setTaskCode(currTask.getTaskCode());
        inStockInfo.setKilnId(currTask.getKilnId());
        inStockInfo.setKilnName(currTask.getKilnName());
        inStockInfo.setLocationId(location.getId());
        inStockInfo.setLocationName(location.getLocationNameAlias());
        inStockInfo.setType(1);
        setCommonField(inStockInfo);
        inStockInfoServiceBiz.save(inStockInfo);
        // TODO: 2021/12/14  调用车辆填入起终点,得到车辆执行结果,填入任务号（待确认）

        if (true){
            //更新关系表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setStatus(2);
            runTask.setEndTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            //更新currTask表
            currTask.setCacheSatatus(2);
            currTask.setUpdateTime(LocalDateTime.now());
            currTaskServiceBiz.updateById(currTask);
            //更新in_stock_info表
            inStockInfo.setStatus(2);
            inStockInfo.setInTime(LocalDateTime.now());
            inStockInfoServiceBiz.updateById(inStockInfo);
            //更新小车状态
            vehicle.setStatus(0);
            vehicleInfoServiceBiz.updateById(vehicle);
        }else {
            runTask.setStatus(3);
            runTask.setUpdateTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            logger.info("任务 "+currTask.getTaskCode()+" 车辆从液压台到缓存区过程中失败。");
            // TODO: 2021/12/14 websocket推送到前端
        }
    }
    //窑炉加工完成，调用车辆入库到缓存区
    @Async("asyncServiceExecutor")
    public void asynRunTaskForKilnToWarehouse(Long currTaskId,Long vehicleId){
        CurrTask currTask = currTaskServiceBiz.getById(currTaskId);
        VehicleInfo vehicle = vehicleInfoServiceBiz.getById(vehicleId);
        Integer otherNowPoint=0;
        if (vehicleId==1){
            // TODO: 2022/2/26 获取RGV2当前位置
            otherNowPoint=1;
        }else {
            //获取RGV1当前位置
            otherNowPoint=2;
        }
        //两辆小车是否冲突
        Boolean conflictBoolean = taskDistanceUtils.conflictForVehicle(vehicleId, currTask.getStartPosition(), currTask.getTargetPosition(), otherNowPoint);
        while (conflictBoolean){
            //若冲突
            // TODO: 2022/2/26  执行小车移位任务。移位任务完成后再判断是否冲突,移位任务可以直接让小车移位到两端
        }
        // TODO: 2021/12/28 location排序
        Location location = locationServiceBiz.list(new QueryWrapper<Location>()
                .eq(Location.STATUS, 1)
                .eq(Location.VALID, 1)).get(0);
        currTask.setOutStartPosition(currTask.getTargetPosition());
        // location对应的点位
        //String outTargetPosition = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, location.getCode())).getCode();
        currTask.setOutTargetPosition(location.getCode());
        currTaskServiceBiz.updateById(currTask);
        //新建一条执行任务的关系表存放任务执行信息,终点为缓存区空闲库位
        RunTask runTask=new RunTask();
        setCommonField(runTask);
        runTask.setTaskId(currTask.getId());
        runTask.setTaskCode(currTask.getTaskCode());
        runTask.setVehicleId(vehicleId);
        runTask.setBeginTime(LocalDateTime.now());
        runTask.setStartPosition(currTask.getOutStartPosition());
        runTask.setEndPosition(currTask.getOutTargetPosition());
        runTaskServiceBiz.save(runTask);
        //更新 location 表对应库位状态为占用
        location.setStatus(1);
        location.setUpdateTime(LocalDateTime.now());
        locationServiceBiz.updateById(location);
        currTask.setOutRunTaskId(runTask.getId());
        currTaskServiceBiz.updateById(currTask);
        //添加一条库位详情到in_stock_info表
        InStockInfo inStockInfo=new InStockInfo();
        inStockInfo.setStatus(1);
        inStockInfo.setInterCode(IDGenerator.gen("HCT", "yyyyMMddHHmm", 2, "WAREHOUSE_CODE"));
        inStockInfo.setTaskId(currTask.getId());
        inStockInfo.setTaskCode(currTask.getTaskCode());
        inStockInfo.setKilnId(currTask.getKilnId());
        inStockInfo.setKilnName(currTask.getKilnName());
        inStockInfo.setLocationId(location.getId());
        inStockInfo.setLocationName(location.getLocationNameAlias());
        inStockInfo.setType(0);
        setCommonField(inStockInfo);
        inStockInfoServiceBiz.save(inStockInfo);
        // TODO: 2021/12/14  调用车辆填入起终点,得到车辆执行结果,填入任务号（待确认）
        if (true){
            //更新关系表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setStatus(2);
            runTask.setEndTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            //更新currTask表
            currTask.setIsOut(2);
            currTask.setUpdateTime(LocalDateTime.now());
            currTask.setStatus(2);
            currTaskServiceBiz.updateById(currTask);
            //更新in_stock_info表
            inStockInfo.setStatus(2);
            inStockInfo.setInTime(LocalDateTime.now());
            inStockInfoServiceBiz.updateById(inStockInfo);
            //更新小车状态
            vehicle.setStatus(0);
            vehicleInfoServiceBiz.updateById(vehicle);
        }else {
            runTask.setStatus(3);
            runTask.setUpdateTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            logger.info("任务 "+currTask.getTaskCode()+" 车辆从窑炉到缓存区过程中失败。");
            // TODO: 2021/12/14 websocket推送到前端
        }
    }
    //出库。起点为缓存区库位，终点为提升台
    @Async("asyncServiceExecutor")
    public void asynRunOutStock(Long runTaskId,Long locationId){
        //修改调度任务状态
        RunTask runTask = runTaskServiceBiz.getById(runTaskId);
        //修改缓存区状态
        InStockInfo inStockInfo = inStockInfoServiceBiz.getOne(new QueryWrapper<InStockInfo>()
                .eq("location_id", locationId));
        inStockInfo.setStatus(1);
        inStockInfoServiceBiz.updateById(inStockInfo);

        // TODO: 2021/12/14  调用车辆填入起终点,得到车辆执行结果,填入任务号（待确认）

        if (true){
            //更新关系表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setStatus(2);
            runTask.setEndTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            //更新in_stock_info表
            inStockInfo.setStatus(2);
            inStockInfo.setInTime(LocalDateTime.now());
            inStockInfoServiceBiz.updateById(inStockInfo);
            //缓存区存储历史
            InStockInfoHis inStockInfoHis=new InStockInfoHis();
            BeanUtils.copyProperties(inStockInfo,inStockInfoHis);
            inStockInfoHisServiceBiz.save(inStockInfoHis);
            inStockInfoServiceBiz.removeById(inStockInfo.getId());
            //更新库位状态
            Location location = locationServiceBiz.getById(locationId);
            location.setStatus(0);
            locationServiceBiz.updateById(location);

        }else {
            runTask.setStatus(3);
            runTask.setUpdateTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            logger.info("任务 "+runTask.getTaskCode()+" 车辆从缓存区到提升平台过程中失败。");
            // TODO: 2021/12/14 websocket推送到前端
        }
    }
    //允许进炉信号为true，记录开始加工时的能源消耗数据，再写定时任务记录加工过程的消耗数据，再写一个异步方法，加工任务结束的时候记录一下各项消耗的值
    public void asynStartRecordConsume(Long currTaskId,Long kilnId){
        // TODO: 2022/2/17
        KilnInfo kilnInfo = kilnInfoServiceBiz.getById(kilnId);
        PlcNameSpace nameSpace = plcNameSpaceServiceBiz.getOne(new QueryWrapper<PlcNameSpace>()
                .eq(PlcNameSpace.EQ_ID, kilnId)
                .eq(PlcNameSpace.EQ_TYPE, 0));
        String nameSpaceNote = nameSpace.getNote();
        switch (kilnInfo.getType()){
            //加工炉
            case 1:
                //实际温度
                Double actualTemp =readPlc(nameSpaceNote,"ActualTemp");
                //设定温度
                Double setUpTemp = readPlc(nameSpaceNote,"SetUpTemp");
                //油槽实际温度
                Double actualOilTemp = readPlc(nameSpaceNote, "ActualOilTemp");
                //油槽设定温度
                Double setUpOilTemp = readPlc(nameSpaceNote,"SetUpOilTemp");
                ParTemValue parTemValue=new ParTemValue();
                parTemValue.setSetTemValue(setUpTemp.floatValue());
                parTemValue.setActTemValue(actualTemp.floatValue());
                parTemValue.setOilTankSetTemValue(setUpOilTemp.floatValue());
                parTemValue.setOilTankActTemValue(actualOilTemp.floatValue());
                parTemValue.setKilnId(kilnId);
                parTemValue.setTaskId(currTaskId);
                setCommonField(parTemValue);
                parTemValueServiceBiz.save(parTemValue);
                //电度值
                Double electricalValue = readPlc(nameSpaceNote,"ElectricalValue");
                //A向电压
                Double phaseAVoltage = readPlc(nameSpaceNote,"PhaseAVoltage");
                Double phaseBVoltage = readPlc(nameSpaceNote, "PhaseBVoltage");
                Double phaseCVoltage = readPlc(nameSpaceNote, "PhaseCVoltage");
                ParEleValue parEleValue=new ParEleValue();
                parEleValue.setAVoltagevValue(phaseAVoltage.floatValue());
                parEleValue.setAVoltagevValue(phaseBVoltage.floatValue());
                parEleValue.setAVoltagevValue(phaseCVoltage.floatValue());
                parEleValue.setTotalBat(electricalValue.floatValue());
                parEleValue.setKilnId(kilnId);
                parEleValue.setTaskId(currTaskId);
                setCommonField(parEleValue);
                parEleValueServiceBiz.save(parEleValue);
                //一号油搅拌转速
                Double oilStirringSpeed1 = readPlc(nameSpaceNote, "OilStirringSpeed1");
                //二号油搅拌转速
                Double oilStirringSpeed2 = readPlc(nameSpaceNote, "OilStirringSpeed2");
                ParRotSpeedValue parRotSpeedValue=new ParRotSpeedValue();
                parRotSpeedValue.setOilStiSpeedAValue(oilStirringSpeed1.floatValue());
                parRotSpeedValue.setOilStiSpeedAValue(oilStirringSpeed2.floatValue());
                parRotSpeedValue.setKilnId(kilnId);
                parRotSpeedValue.setTaskId(currTaskId);
                setCommonField(parRotSpeedValue);
                parRotSpeedValueServiceBiz.save(parRotSpeedValue);
                //氮气
                Double nitrogenFlow = readPlc(nameSpaceNote, "NitrogenFlow");
                //甲烷
                Double methanolFlow = readPlc(nameSpaceNote, "MethanolFlow");
                //丙烷
                Double propaneFlow = readPlc(nameSpaceNote, "PropaneFlow");
                //氨气
                Double ammoniaFlow = readPlc(nameSpaceNote, "AmmoniaFlow");
                //实际碳势
                Double actualCarbon = readPlc(nameSpaceNote, "ActualCarbon");
                //设定碳势
                Double setupCarbon = readPlc(nameSpaceNote, "SetupCarbon");
                ParGasValue parGasValue=new ParGasValue();
                parGasValue.setNitFlowValue(nitrogenFlow.floatValue());
                parGasValue.setAmmoniaFlowValue(ammoniaFlow.floatValue());
                parGasValue.setMethanolFlow(methanolFlow.floatValue());
                parGasValue.setPropaneFlow(propaneFlow.floatValue());
                parGasValue.setActualNitPotValue(actualCarbon.floatValue());
                parGasValue.setSetNitPotValue(setupCarbon.floatValue());
                parGasValue.setKilnId(kilnId);
                parGasValue.setTaskId(currTaskId);
                setCommonField(parGasValue);
                parGasValueServiceBiz.save(parGasValue);
                break;
            //回火炉
            case 2:
                //实际温度
                actualTemp =readPlc(nameSpaceNote,"ActualTemp");
                //设定温度
                setUpTemp = readPlc(nameSpaceNote,"SetUpTemp");
                ParTemValue parTemValue2=new ParTemValue();
                parTemValue2.setSetTemValue(setUpTemp.floatValue());
                parTemValue2.setActTemValue(actualTemp.floatValue());
                parTemValue2.setKilnId(kilnId);
                parTemValue2.setTaskId(currTaskId);
                setCommonField(parTemValue2);
                parTemValueServiceBiz.save(parTemValue2);
                //电度值
                electricalValue = readPlc(nameSpaceNote,"TotalElectricity");
                //A向电压
                phaseAVoltage = readPlc(nameSpaceNote,"PhaseAVoltage");
                phaseBVoltage = readPlc(nameSpaceNote, "PhaseBVoltage");
                phaseCVoltage = readPlc(nameSpaceNote, "PhaseCVoltage");
                ParEleValue parEleValue2=new ParEleValue();
                parEleValue2.setAVoltagevValue(phaseAVoltage.floatValue());
                parEleValue2.setAVoltagevValue(phaseBVoltage.floatValue());
                parEleValue2.setAVoltagevValue(phaseCVoltage.floatValue());
                parEleValue2.setTotalBat(electricalValue.floatValue());
                parEleValue2.setKilnId(kilnId);
                parEleValue2.setTaskId(currTaskId);
                setCommonField(parEleValue2);
                parEleValueServiceBiz.save(parEleValue2);
                break;
            //氮化炉
            case 3:
                //实际温度
                actualTemp =readPlc(nameSpaceNote,"ActualTemp");
                //设定温度
                setUpTemp = readPlc(nameSpaceNote,"SetUpTemp");
                //炉压
                Double furnacePressure = readPlc(nameSpaceNote, "FurnacePressure");
                //外一区温度
                Double outerZoneITemperature = readPlc(nameSpaceNote, "OuterZoneITemperature");
                //外二区温度
                Double outerZone2Temperature = readPlc(nameSpaceNote, "OuterZone2Temperature");
                ParTemValue parTemValue3=new ParTemValue();
                parTemValue3.setSetTemValue(setUpTemp.floatValue());
                parTemValue3.setActTemValue(actualTemp.floatValue());
                parTemValue3.setOuterZone1Temp(outerZoneITemperature.floatValue());
                parTemValue3.setOuterZone2Temp(outerZone2Temperature.floatValue());
                parTemValue3.setFurnacePressure(furnacePressure.floatValue());
                parTemValue3.setKilnId(kilnId);
                parTemValue3.setTaskId(currTaskId);
                setCommonField(parTemValue3);
                parTemValueServiceBiz.save(parTemValue3);
                //电度值
                electricalValue = readPlc(nameSpaceNote,"ElectricalValue");
                //A向电压
                phaseAVoltage = readPlc(nameSpaceNote,"PhaseAVoltage");
                phaseBVoltage = readPlc(nameSpaceNote, "PhaseBVoltage");
                phaseCVoltage = readPlc(nameSpaceNote, "PhaseCVoltage");
                ParEleValue parEleValue3=new ParEleValue();
                parEleValue3.setAVoltagevValue(phaseAVoltage.floatValue());
                parEleValue3.setAVoltagevValue(phaseBVoltage.floatValue());
                parEleValue3.setAVoltagevValue(phaseCVoltage.floatValue());
                parEleValue3.setTotalBat(electricalValue.floatValue());
                parEleValue3.setKilnId(kilnId);
                parEleValue3.setTaskId(currTaskId);
                setCommonField(parEleValue3);
                parEleValueServiceBiz.save(parEleValue3);
                //实际氮势
                actualCarbon = readPlc(nameSpaceNote, "ActualCarbon");
                //设定氮势
                setupCarbon = readPlc(nameSpaceNote, "SetupCarbon");
                //氢含量
                Double hydrogenContent = readPlc(nameSpaceNote, "HydrogenContent");
                //分解率
                Double decompositionRate = readPlc(nameSpaceNote, "DecompositionRate");
                //氨气量
                ammoniaFlow=readPlc(nameSpaceNote,"AmmoniaFlow");
                //氮气量
                nitrogenFlow=readPlc(nameSpaceNote,"NitrogenFlow");
                //二氧化碳量
                Double carbonDioxideFlow = readPlc(nameSpaceNote, "CarbonDioxideFlow");
                ParGasValue parGasValue3=new ParGasValue();
                parGasValue3.setNitFlowValue(nitrogenFlow.floatValue());
                parGasValue3.setAmmoniaFlowValue(ammoniaFlow.floatValue());
                parGasValue3.setActualNitPotValue(actualCarbon.floatValue());
                parGasValue3.setSetNitPotValue(setupCarbon.floatValue());
                parGasValue3.setHydrogenContent(hydrogenContent.floatValue());
                parGasValue3.setDecompositionRate(decompositionRate.floatValue());
                parGasValue3.setKilnId(kilnId);
                parGasValue3.setTaskId(currTaskId);
                setCommonField(parGasValue3);
                parGasValueServiceBiz.save(parGasValue3);
                break;
            //清洗炉
            case 4:
                //电度值
                electricalValue = readPlc(nameSpaceNote,"TotalElectricity");
                //A向电压
                phaseAVoltage = readPlc(nameSpaceNote,"PhaseAVoltage");
                phaseBVoltage = readPlc(nameSpaceNote, "PhaseBVoltage");
                phaseCVoltage = readPlc(nameSpaceNote, "PhaseCVoltage");
                ParEleValue parEleValue4=new ParEleValue();
                parEleValue4.setAVoltagevValue(phaseAVoltage.floatValue());
                parEleValue4.setAVoltagevValue(phaseBVoltage.floatValue());
                parEleValue4.setAVoltagevValue(phaseCVoltage.floatValue());
                parEleValue4.setTotalBat(electricalValue.floatValue());
                parEleValue4.setKilnId(kilnId);
                parEleValue4.setTaskId(currTaskId);
                setCommonField(parEleValue4);
                parEleValueServiceBiz.save(parEleValue4);
                break;
            default:
                logger.info("任务："+currTaskId+"开始在"+kilnInfo.getKilnName()+"加工，能源消耗开始统计。");
        }
    }
    //允许出炉信号为true，记录结束加工时的能源消耗数据
    public void asynEndRecordConsume(Long currTaskId,Long kilnId){
        // TODO: 2022/2/17
        KilnInfo kilnInfo = kilnInfoServiceBiz.getById(kilnId);
        PlcNameSpace nameSpace = plcNameSpaceServiceBiz.getOne(new QueryWrapper<PlcNameSpace>()
                .eq(PlcNameSpace.EQ_ID, kilnId)
                .eq(PlcNameSpace.EQ_TYPE, 0));
        String nameSpaceNote = nameSpace.getNote();
        switch (kilnInfo.getType()){
            //加工炉
            case 1:
                //实际温度
                Double actualTemp =readPlc(nameSpaceNote,"ActualTemp");
                //设定温度
                Double setUpTemp = readPlc(nameSpaceNote,"SetUpTemp");
                //油槽实际温度
                Double actualOilTemp = readPlc(nameSpaceNote, "ActualOilTemp");
                //油槽设定温度
                Double setUpOilTemp = readPlc(nameSpaceNote,"SetUpOilTemp");
                ParTemValue parTemValue=new ParTemValue();
                parTemValue.setSetTemValue(setUpTemp.floatValue());
                parTemValue.setActTemValue(actualTemp.floatValue());
                parTemValue.setOilTankSetTemValue(setUpOilTemp.floatValue());
                parTemValue.setOilTankActTemValue(actualOilTemp.floatValue());
                parTemValue.setKilnId(kilnId);
                parTemValue.setTaskId(currTaskId);
                setCommonField(parTemValue);
                parTemValueServiceBiz.save(parTemValue);
                //电度值
                Double electricalValue = readPlc(nameSpaceNote,"ElectricalValue");
                //A向电压
                Double phaseAVoltage = readPlc(nameSpaceNote,"PhaseAVoltage");
                Double phaseBVoltage = readPlc(nameSpaceNote, "PhaseBVoltage");
                Double phaseCVoltage = readPlc(nameSpaceNote, "PhaseCVoltage");
                ParEleValue parEleValue=new ParEleValue();
                parEleValue.setAVoltagevValue(phaseAVoltage.floatValue());
                parEleValue.setAVoltagevValue(phaseBVoltage.floatValue());
                parEleValue.setAVoltagevValue(phaseCVoltage.floatValue());
                parEleValue.setTotalBat(electricalValue.floatValue());
                parEleValue.setKilnId(kilnId);
                parEleValue.setTaskId(currTaskId);
                setCommonField(parEleValue);
                parEleValueServiceBiz.save(parEleValue);
                //一号油搅拌转速
                Double oilStirringSpeed1 = readPlc(nameSpaceNote, "OilStirringSpeed1");
                //二号油搅拌转速
                Double oilStirringSpeed2 = readPlc(nameSpaceNote, "OilStirringSpeed2");
                ParRotSpeedValue parRotSpeedValue=new ParRotSpeedValue();
                parRotSpeedValue.setOilStiSpeedAValue(oilStirringSpeed1.floatValue());
                parRotSpeedValue.setOilStiSpeedAValue(oilStirringSpeed2.floatValue());
                parRotSpeedValue.setKilnId(kilnId);
                parRotSpeedValue.setTaskId(currTaskId);
                setCommonField(parRotSpeedValue);
                parRotSpeedValueServiceBiz.save(parRotSpeedValue);
                //氮气
                Double nitrogenFlow = readPlc(nameSpaceNote, "NitrogenFlow");
                //甲烷
                Double methanolFlow = readPlc(nameSpaceNote, "MethanolFlow");
                //丙烷
                Double propaneFlow = readPlc(nameSpaceNote, "PropaneFlow");
                //氨气
                Double ammoniaFlow = readPlc(nameSpaceNote, "AmmoniaFlow");
                //实际碳势
                Double actualCarbon = readPlc(nameSpaceNote, "ActualCarbon");
                //设定碳势
                Double setupCarbon = readPlc(nameSpaceNote, "SetupCarbon");
                ParGasValue parGasValue=new ParGasValue();
                parGasValue.setNitFlowValue(nitrogenFlow.floatValue());
                parGasValue.setAmmoniaFlowValue(ammoniaFlow.floatValue());
                parGasValue.setMethanolFlow(methanolFlow.floatValue());
                parGasValue.setPropaneFlow(propaneFlow.floatValue());
                parGasValue.setActualNitPotValue(actualCarbon.floatValue());
                parGasValue.setSetNitPotValue(setupCarbon.floatValue());
                parGasValue.setKilnId(kilnId);
                parGasValue.setTaskId(currTaskId);
                setCommonField(parGasValue);
                parGasValueServiceBiz.save(parGasValue);
                break;
            //回火炉
            case 2:
                //实际温度
                actualTemp =readPlc(nameSpaceNote,"ActualTemp");
                //设定温度
                setUpTemp = readPlc(nameSpaceNote,"SetUpTemp");
                ParTemValue parTemValue2=new ParTemValue();
                parTemValue2.setSetTemValue(setUpTemp.floatValue());
                parTemValue2.setActTemValue(actualTemp.floatValue());
                parTemValue2.setKilnId(kilnId);
                parTemValue2.setTaskId(currTaskId);
                setCommonField(parTemValue2);
                parTemValueServiceBiz.save(parTemValue2);
                //电度值
                electricalValue = readPlc(nameSpaceNote,"TotalElectricity");
                //A向电压
                phaseAVoltage = readPlc(nameSpaceNote,"PhaseAVoltage");
                phaseBVoltage = readPlc(nameSpaceNote, "PhaseBVoltage");
                phaseCVoltage = readPlc(nameSpaceNote, "PhaseCVoltage");
                ParEleValue parEleValue2=new ParEleValue();
                parEleValue2.setAVoltagevValue(phaseAVoltage.floatValue());
                parEleValue2.setAVoltagevValue(phaseBVoltage.floatValue());
                parEleValue2.setAVoltagevValue(phaseCVoltage.floatValue());
                parEleValue2.setTotalBat(electricalValue.floatValue());
                parEleValue2.setKilnId(kilnId);
                parEleValue2.setTaskId(currTaskId);
                setCommonField(parEleValue2);
                parEleValueServiceBiz.save(parEleValue2);
                break;
            //氮化炉
            case 3:
                //实际温度
                actualTemp =readPlc(nameSpaceNote,"ActualTemp");
                //设定温度
                setUpTemp = readPlc(nameSpaceNote,"SetUpTemp");
                //炉压
                Double furnacePressure = readPlc(nameSpaceNote, "FurnacePressure");
                //外一区温度
                Double outerZoneITemperature = readPlc(nameSpaceNote, "OuterZoneITemperature");
                //外二区温度
                Double outerZone2Temperature = readPlc(nameSpaceNote, "OuterZone2Temperature");
                ParTemValue parTemValue3=new ParTemValue();
                parTemValue3.setSetTemValue(setUpTemp.floatValue());
                parTemValue3.setActTemValue(actualTemp.floatValue());
                parTemValue3.setOuterZone1Temp(outerZoneITemperature.floatValue());
                parTemValue3.setOuterZone2Temp(outerZone2Temperature.floatValue());
                parTemValue3.setFurnacePressure(furnacePressure.floatValue());
                parTemValue3.setKilnId(kilnId);
                parTemValue3.setTaskId(currTaskId);
                setCommonField(parTemValue3);
                parTemValueServiceBiz.save(parTemValue3);
                //电度值
                electricalValue = readPlc(nameSpaceNote,"ElectricalValue");
                //A向电压
                phaseAVoltage = readPlc(nameSpaceNote,"PhaseAVoltage");
                phaseBVoltage = readPlc(nameSpaceNote, "PhaseBVoltage");
                phaseCVoltage = readPlc(nameSpaceNote, "PhaseCVoltage");
                ParEleValue parEleValue3=new ParEleValue();
                parEleValue3.setAVoltagevValue(phaseAVoltage.floatValue());
                parEleValue3.setAVoltagevValue(phaseBVoltage.floatValue());
                parEleValue3.setAVoltagevValue(phaseCVoltage.floatValue());
                parEleValue3.setTotalBat(electricalValue.floatValue());
                parEleValue3.setKilnId(kilnId);
                parEleValue3.setTaskId(currTaskId);
                setCommonField(parEleValue3);
                parEleValueServiceBiz.save(parEleValue3);
                //实际氮势
                actualCarbon = readPlc(nameSpaceNote, "ActualCarbon");
                //设定氮势
                setupCarbon = readPlc(nameSpaceNote, "SetupCarbon");
                //氢含量
                Double hydrogenContent = readPlc(nameSpaceNote, "HydrogenContent");
                //分解率
                Double decompositionRate = readPlc(nameSpaceNote, "DecompositionRate");
                //氨气量
                ammoniaFlow=readPlc(nameSpaceNote,"AmmoniaFlow");
                //氮气量
                nitrogenFlow=readPlc(nameSpaceNote,"NitrogenFlow");
                //二氧化碳量
                Double carbonDioxideFlow = readPlc(nameSpaceNote, "CarbonDioxideFlow");
                ParGasValue parGasValue3=new ParGasValue();
                parGasValue3.setNitFlowValue(nitrogenFlow.floatValue());
                parGasValue3.setAmmoniaFlowValue(ammoniaFlow.floatValue());
                parGasValue3.setActualNitPotValue(actualCarbon.floatValue());
                parGasValue3.setSetNitPotValue(setupCarbon.floatValue());
                parGasValue3.setHydrogenContent(hydrogenContent.floatValue());
                parGasValue3.setDecompositionRate(decompositionRate.floatValue());
                parGasValue3.setKilnId(kilnId);
                parGasValue3.setTaskId(currTaskId);
                setCommonField(parGasValue3);
                parGasValueServiceBiz.save(parGasValue3);
                break;
            //清洗炉
            case 4:
                //电度值
                electricalValue = readPlc(nameSpaceNote,"TotalElectricity");
                //A向电压
                phaseAVoltage = readPlc(nameSpaceNote,"PhaseAVoltage");
                phaseBVoltage = readPlc(nameSpaceNote, "PhaseBVoltage");
                phaseCVoltage = readPlc(nameSpaceNote, "PhaseCVoltage");
                ParEleValue parEleValue4=new ParEleValue();
                parEleValue4.setAVoltagevValue(phaseAVoltage.floatValue());
                parEleValue4.setAVoltagevValue(phaseBVoltage.floatValue());
                parEleValue4.setAVoltagevValue(phaseCVoltage.floatValue());
                parEleValue4.setTotalBat(electricalValue.floatValue());
                parEleValue4.setKilnId(kilnId);
                parEleValue4.setTaskId(currTaskId);
                setCommonField(parEleValue4);
                parEleValueServiceBiz.save(parEleValue4);
                break;
            default:
                logger.info("任务："+currTaskId+"开始在"+kilnInfo.getKilnName()+"加工，能源消耗开始统计。");
        }
    }
    /**
     * 获取变量值
     * @param nameSpace nameSpace前缀
     * @param variableName 变量名称
     * @return 变量值
     */
    private Double readPlc(String nameSpace, String variableName){
        String identifier=nameSpace.concat(variableName);
        Map<String, Object> json = new HashMap();
        json.put("nameSpace", 6);
        json.put("plcName", "plc1");
        json.put("identifier",identifier);
        String result = null;
        try {
            result = HttpClient.httpPost("http://localhost:8009/opcua/read", JSON.toJSONString(json));
        } catch (IOException e) {
            e.printStackTrace();
            logger.info("能源消耗统计：获取 "+identifier+" 的值失败");
        }
         return Double.valueOf(JSONObject.parseObject(result).get("result").toString());
    }
}
